#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/ovm/core'

module Puppet
  # Type responsible for managing of ovm file servers available for manager
  Type.newtype(:ovm_file_server) do
    include Puppet_X::EnterpriseModules::Ovm::Type

    desc <<-EOD
    This resource allows you manage Oracle VM File Server.
    Here is an example on how to use this:

        ovm_file_server { 'default/nfs_server_1':
          ensure          => 'present',
          access_host     => '192.168.56.3',
          admin_servers   => ['ovs.example.com']
          uniform_exports => 'true',
        }

    EOD

    restfull_type('FileServer')
    exclude_properties(:admin_servers, :refresh_servers, :plugin)

    def create_payload
      admin_ids = server_data.select {|s| admin_servers.include?(s['id']['name'])}
      super.merge(
        'fileServerPluginId' => file_server_plugin_id_for(plugin),
        'adminServerIds'     =>  admin_ids.collect {|s| s['id']},
      )
    end

    after_create do
      refresh_server
      refresh_file_systems
    end

    def server_refresh_path
      "#{::Puppet_X::EnterpriseModules::Ovm::Type::BASE_PATH}/FileServer/#{file_server_id}/refresh?fileServerId=#{file_server_id}"
    end

    def file_system_refresh_path(fs_id)
      "#{::Puppet_X::EnterpriseModules::Ovm::Type::BASE_PATH}/FileSystem/#{fs_id}/refresh?fileSystemId=#{fs_id}"
    end

    def refresh_server
      Puppet.debug "Post a refresh for file_system #{file_server_name}."
      with_manager_connection do |connection|
        connection.validated(:put, server_refresh_path, {})
      end
    end

    def refresh_file_systems
      file_system_data.each do | fs_data|
        next unless fs_data['fileServerIds'].collect {|e| e['value']}.include?(file_server_id)
        fs_id = fs_data.dig('id','value')
        Puppet.debug "Post a refresh for file_system #{fs_data.dig('id','name')}."
        with_manager_connection do |connection|
          connection.validated(:put,file_system_refresh_path(fs_id), {})
        end
      end
    end

    ensurable

    parameter :id
    parameter :manager_name
    parameter :name
    parameter :file_server_name
    parameter :plugin

    property  :access_host
    property  :description
    property  :uniform_exports
    property  :admin_servers
    property  :refresh_servers

    add_title_attributes(:file_server_name)
  end
end
