#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/ovm/core'

module Puppet
  Type.newtype(:ovm_storage_array) do
    include Puppet_X::EnterpriseModules::Ovm::Type

    desc <<-EOD
    This resource allows you manage Oracle VM Storage Array.
    Here is an example on how to use this:

        ovm_storage_array { 'default/Unmanaged FibreChannel Storage Array':
          ensure                  => 'present',
          storage_type            => 'ISCSI',
          admin_password          => '****************',
        }

    EOD

    restfull_type('StorageArray')
    exclude_properties(:admin_servers, :plugin)

    #
    # We don't want puppet to manage Unmanaged resources
    #
    resource_filter do |resource|
      resource['name'] !~ /unmanaged/i
    end

    def create_payload
      admin_ids = server_data.select {|s| admin_servers.include?(s['id']['name'])}
      super.merge(
        'storageType'          => storage_type,
        'storageArrayPluginId' => storage_array_plugin_id_for(plugin),
        'adminServerIds'       => admin_ids.collect { |s| s['id'] },
      )
    end

    after_create do
      validate_server
      refresh_server
    end

    def server_validate_path
      "#{::Puppet_X::EnterpriseModules::Ovm::Type::BASE_PATH}/StorageArray/#{storage_array_id}/validate?storageArrayId=#{storage_array_id}"
    end

    def validate_server
      Puppet.debug "Post a validate for storage array #{storage_array_name}."
      with_manager_connection do |connection|
        connection.validated(:put, server_validate_path, {})
      end
    end

    def server_refresh_path
      "#{::Puppet_X::EnterpriseModules::Ovm::Type::BASE_PATH}/StorageArray/#{storage_array_id}/refresh?storageArrayId=#{storage_array_id}"
    end

    def refresh_server
      Puppet.debug "Post a refresh for storage array #{storage_array_name}."
      with_manager_connection do |connection|
        connection.validated(:put, server_refresh_path, {})
      end
    end

    ensurable

    parameter :id
    parameter :manager_name
    parameter :name
    parameter :storage_array_name
    parameter :plugin

    property  :description
    property  :storage_type
    property  :access_hosts
    property  :use_chap
    property  :admin_hostname
    property  :admin_username
    property  :admin_password
    property  :extra_information
    property  :storage_name
    property  :admin_servers

    add_title_attributes(:storage_array_name)
  end
end
