#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/ovm/core'

module Puppet
  Type.newtype(:ovm_virtual_disk) do
    include Puppet_X::EnterpriseModules::Ovm::Type
    include Puppet_X::EnterpriseModules::Ovm::Data

    desc <<-EOD
    This resource allows you manage Oracle VM Virtual Disk.
    Here is an example on how to use this:

        ovm_virtual_disk { 'default/VirtualDisk1':
          ensure      => 'present',
          description => 'First Virtual Disk',
        }

    Here is an example on how to add iso to repository:

        ovm_virtual_disk { 'default/repo_1/CentOS 7 minimal iso':
          ensure      => 'present',
          description => 'minimal image of CentOS 7',
          disk_type   => 'VIRTUAL_CDROM',
          shareable   => true,
          urls        => ['http://ftp.pbone.net/pub/centos/7/isos/x86_64/CentOS-7-x86_64-Minimal-1708.iso'],
          async       => false,
        }

    EOD

    restfull_type('VirtualDisk')
    exclude_properties(:sparse, :urls, :proxy, :async, :disk_type)

    ensurable

    parameter :id
    parameter :manager_name
    parameter :name
    parameter :repository_name
    parameter :virtual_disk_name

    #
    # Excluded from resource, parameters issued only at creation.
    #
    parameter :sparse
    parameter :urls
    parameter :async
    parameter :proxy

    #
    # Manageable properties.
    #
    property  :description
    property  :shareable
    property  :size
    property  :disk_type

    add_title_attributes(:repository_name, :virtual_disk_name) { /(.+)\/(.+)/ }

    #
    # Empty CDRom is a default device ALWAYS available. Don't allow
    # puppet to see or manage it.
    #
    resource_filter do |resource|
      resource['name'] != 'Empty CDROM'
    end

    def create_path
      virtual_cdrom? ? super() : \
        "#{::Puppet_X::EnterpriseModules::Ovm::Type::BASE_PATH}/Repository/#{repository_id}/VirtualDisk?sparse=#{sparse}&size=#{size}"
    end

    def destroy_path
      virtual_cdrom? ? super() : \
        "#{::Puppet_X::EnterpriseModules::Ovm::Type::BASE_PATH}/Repository/#{repository_id}/VirtualDisk?sparse=#{sparse}&size=#{size}"
    end

    on_create do
      fail 'Specify disk_type at creation of ovm_virtual_disk' if disk_type.nil?
      virtual_cdrom? ? add_iso : super()
    end

    #
    # This block executes params which cannot be populated at the creation of
    # virtual disk.
    #
    after_create do
      modify_iso if disk_type == 'VIRTUAL_CDROM'
    end

    on_destroy do
      Puppet.info "destroy #{name}"
      with_manager_connection do |connection|
        virtual_disk_id = virtual_disk_id_for(virtual_disk_name)['value']
        repository_id   = repository_id_for(repository_name)['value']
        path = "/ovm/core/wsapi/rest/Repository/#{repository_id}/VirtualDisk/#{virtual_disk_id}"
        connection.validated(:delete, path)
      end
      nil
    end

    private

    def virtual_cdrom?
      disk_type == 'VIRTUAL_CDROM'
    end

    def add_iso
      with_manager_connection do |connection|
        path = "#{::Puppet_X::EnterpriseModules::Ovm::Type::BASE_PATH}/Repository/#{repository_id}/importVirtualDisk?diskType=VIRTUAL_CDROM"
        #
        # Create repository import object that is issued at creation of iso.
        #
        payload = Hash.new.tap do |repository_import|
          repository_import['urls']  = urls
          repository_import['proxy'] = proxy if proxy
        end
        #
        # If async is set to true don't validate end of the job.
        #
        if async == true
          connection.put(path, payload)
        else
          connection.validated(:put, path, payload)
        end
      end
      nil
    end

    def modify_iso
      return Puppet.notice "Changing name of the VIRTUAL_CDROM is disabled when async is on" if async
      #
      # After addition of the VIRTUAL_CDROM the OVM Server adds description like
      # "Import URL: http://url_here.com". Code searches for the virtual_disk
      # object based on description content.
      #
      virtual_disk_object = virtual_disk_data.find do |d|
        d['description'] =~ Regexp.new(urls.first.to_s)
      end
      fail 'Virtual disk object not found after creation' unless virtual_disk_object
      virtual_disk_id = virtual_disk_object['id']
      #
      # Prepare payload for changing the virtual_disk object.
      # Overwrite current object.
      #
      payload = json_body(normalized_data, virtual_disk_object)
      with_manager_connection do |connection|
        path = "#{::Puppet_X::EnterpriseModules::Ovm::Type::BASE_PATH}/VirtualDisk/#{virtual_disk_id['value']}"
        connection.validated(:put, path, payload)
      end
    end
  end
end
