#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/ovm/core'

module Puppet
  # Type responsible for managing of ovm vitualmachines running in serverpools
  # available for manager
  Type.newtype(:ovm_vm) do
    include Puppet_X::EnterpriseModules::Ovm::Type
    include Puppet_X::EnterpriseModules::Ovm::Data

    desc <<-EOD
    This resource allows you manage VM running on Server.
    Here is an example on how to use this:

        ovm_vm { 'default/repo_1/server_pool_1/ovs.example.com/myvm1':
          ensure                      => 'present',
          boot_order                  => ['PXE', 'DISK', 'CDROM'],
          cpu_count                   => '2',
          cpu_count_limit             => '2',
          cpu_priority                => '75',
          cpu_utilization_cap         => '100',
          description                 => 'Test test',
          high_availability           => 'false',
          keymap_name                 => 'en-us',
          memory                      => '1024',
          memory_limit                => '1024',
          os_type                     => 'Cent OS 7',
          restart_action_on_crash     => 'RESTART',
          vm_domain_type              => 'XEN_HVM',
          vm_mouse_type               => 'OS_DEFAULT',
          vm_run_state                => 'STOPPED',
          vm_start_policy             => 'CURRENT_SERVER',
          network_adapters            => { 'adapter_1' => { 'ipAddresses' => [ {"type" => "IPV4", "configType" => "STATIC", "address" => "192.168.56.11", "netmask" => "255.255.255.0" }] } }
        }

    EOD

    restfull_type('Vm')
    exclude_properties(:virtual_disks)

    ensurable

    parameter :id
    parameter :manager_name
    parameter :repository_name
    parameter :name
    parameter :server_pool_name
    parameter :server_name
    parameter :vm_name

    #
    # Managable propeties via REST PUT
    #
    property  :boot_order
    property  :cpu_count
    property  :cpu_count_limit
    property  :cpu_priority
    property  :cpu_utilization_cap
    property  :description
    property  :high_availability
    property  :keymap_name
    property  :memory
    property  :memory_limit
    property  :network_install_path
    property  :os_type
    property  :vm_domain_type
    property  :vm_mouse_type
    property  :vm_start_policy

    # Properties populated only after creation
    property :restart_action_on_crash
    property :virtual_disks
    property :network_adapters
    #
    # This is the latest property. It must be run after all other
    # stuff has been done.
    #
    property :vm_run_state

    def create_payload
      payload = super
      # This params cannot be issued at creation and will be executed in the
      # after_create block.

      prohibited_at_creation = [
        :osVersion,
        :restartActionOnCrash,
        :restartActionOnRestart,
        :networkAdapters
      ].freeze
      payload = payload.reject { |key| prohibited_at_creation.map(&:to_s).include?(key) }

      payload.merge!(
        'repositoryId' => repository_reference,
        'serverPoolId' => server_pool_reference,
        'name'         => vm_name
      )
      payload
    end

    # This block executes params which cannot be populated at the creation of vm
    after_create do
      with_manager_connection do |connection|
        vm_data = vm_data(vm_id)
        payload = json_body(normalized_data, vm_data)
        payload = payload.reject {|key| key == 'networkAdapters' } # Remove Network Adapter
        path = "#{::Puppet_X::EnterpriseModules::Ovm::Type::BASE_PATH}/Vm/#{vm_id}"
        connection.validated(:put, path, payload)
      end
    end

    add_title_attributes(:repository_name, :server_pool_name, :server_name, :vm_name) { /(.+)\/(.+)\/(.+)\/(.+)/ }
  end
end
