#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:virtual_disks,
            :array_matching => :all,
            :parent => Puppet::Parameter::OvmProperty) do
  include EasyType::ArrayProperty
  include Puppet_X::EnterpriseModules::Ovm::Data

  desc <<-EOD
  Specifies the Virtual Disks of Oracle Virtual Machine.
  Here is example how to do this:

      ovm_vm { 'default/mypool1/ovmserver1.example.com:myvm2':
        ...
        virtual_disks => ['0:repo01/vm01_disk']
        ...
      }

  EOD

  validate do |value|
    matches = value.match(/(?<slot>\d+)\:(?<repo>\S+)\/(?<disk>\S+)/)
    fail "#{value} is an invalid format for virtual disks. Use 'slot:repo/disk'." if matches.nil?
  end

  def self.translate_to_resource(raw_resource)
    manager = raw_resource.fetch('manager')
    value = []
    raw_resource['vmDiskMappingIds'].each do |map|
      map_info = vm_disk_mapping_data(map['value'], manager: manager)
      if iso_or_vm_disk?(map_info)
        disk_id   = map_info.dig('virtualDiskId', 'value')
        disk_slot = map_info['diskTarget']
        disk_info = virtual_disk_data(disk_id, manager: manager)
        repo_info = repository_data(disk_info['repositoryId']['value'], manager: manager)
        value << "#{disk_slot}:#{repo_info['name']}/#{disk_info['name']}"
      elsif physical_disk?(map_info)
        disk_id   = map_info.dig('storageElementId', 'value')
        disk_slot = map_info['diskTarget']
        disk_info = storage_element_data(disk_id, manager: manager)
        repo_info = 'unknown' # repository_data(disk_info['repositoryId']['value'], manager: manager)
        value << "#{disk_slot}:physical/#{disk_info['name']}"
      else
        fail 'unsupported disk type found'
      end
    end
    value.sort
  end

  on_apply do
    #
    # For now we work with 3 disk slots.
    #
    max_slots = 3
    #
    # TODO: Current implementation just selects on disk names. If the same disk is in
    # multiple repositories. Things go wrong
    #
    vm_id     = vm_id_for(resource.vm_name)
    @mappings = vm_disk_mapping_data.select {|m| m.dig('vmId','value') == vm_id['value']}
    #
    # If we move a disk, it must first be removed before we can add it again. So we
    # process the disk mappings twice. First we do all remove operations then we
    # do all additions
    #
    (0..max_slots).each do |slot|
      slot_value    = value_for_slot(slot)
      mapping_value = mapping_for_slot(slot)
      if slot_value.nil? && mapping_value.nil?
        # No mapping requested and none exist
        next
      elsif slot_value.nil? && mapping_value
        # A mapping requested, but none exists
        remove_mapping(vm_id['value'], mapping_value.dig('id', 'value'))
      elsif slot_value && mapping_value.nil?
        # A mapping requested, but none exists
        _, repo, disk = slot_value
        add_mapping(slot, vm_id, repo, disk)
      else
        # A mapping exists and one is requested
        _, repo, disk = slot_value
        next if mapping_value.dig('virtualDiskId','name') == disk # Same
        remove_mapping(vm_id['value'], mapping_value.dig('id', 'value'))
        add_mapping(slot, vm_id, repo, disk)
      end
    end
  end

  private

  def self.iso_or_vm_disk?(map_info)
    not map_info['virtualDiskId'].nil?
  end

  def self.physical_disk?(map_info)
    not map_info['storageElementId'].nil?
  end

  def mapping_for_slot(slot)
    @mappings.find { |m| m['diskTarget'] == slot.to_i }
  end

  def remove_mapping(vm_id, mapping_id)
    with_manager_connection do |connection|
      connection.validated(:delete, "/ovm/core/wsapi/rest/Vm/#{vm_id}/VmDiskMapping/#{mapping_id}")
    end
  end

  def add_mapping(slot, vm_id, repo, disk)
    disk_element = case repo
                   when 'physical' then
                     { 'storageElementId' => storage_element_id_for(disk) }
                   else
                     { 'virtualDiskId' => virtual_disk_id_for(disk) }
                   end
    payload = disk_element.merge(
      'vmId'          => vm_id,
      'diskTarget'    => slot
    )
    parameters = URI.encode_www_form('vmId' => vm_id['value'])
    with_manager_connection do |connection|
      connection.validated(:post, "/ovm/core/wsapi/rest/Vm/#{vm_id['value']}/VmDiskMapping?#{parameters}", payload)
    end
  end

  def value_for_slot(slot)
    value.each do | virtual_disk |
      matches = virtual_disk.match(/(?<slot>\d+)\:(?<repo>\S+)\/(?<disk>.+)/)
      return matches['slot'], matches['repo'], matches['disk'] if matches['slot'].to_i == slot.to_i
    end
    nil
  end
end
