#
# See the file "LICENSE" for the full license governing this code.
#
require 'net/https'
require 'uri'
require 'ostruct'
require 'json'

module Puppet_X
  module EnterpriseModules
    class Connection
      include EasyType::Encryption

      DEFAULT_ENDPOINT = 'https://specify.your.host.com/'.freeze
      DEFAULT_USERNAME = 'admin'.freeze
      DEFAULT_PORT     = 7002
      MESSAGE_HEADERS = { 'Content-Type' => 'application/json',
                          'Accept'       => 'application/json' }.freeze

      VERB_MAP = {
        :get    => Net::HTTP::Get,
        :post   => Net::HTTP::Post,
        :put    => Net::HTTP::Put,
        :delete => Net::HTTP::Delete
      }

      VALID_OPTIONS = [:endpoint, :username, :password, :verify_ssl, :port].freeze

      def initialize(options)
        option_fail_message(options) unless all_valid_options?(options)
        endpoint   = options.fetch(:endpoint) { DEFAULT_ENDPOINT }
        port       = options.fetch(:port)     { DEFAULT_PORT }
        @username  = options.fetch(:username) { DEFAULT_USERNAME }
        @password  = authorization_password(options)
        uri = URI.parse(endpoint)
        @http = Net::HTTP.new(uri.host, port)
        @http.use_ssl     = true if uri.scheme == 'https'
        @http.verify_mode = OpenSSL::SSL::VERIFY_NONE if ssl_verification_disabled?(options, uri)
      end

      def ssl_verification_disabled?(options, uri)
        verify_ssl = options.fetch(:verify_ssl) { :true } == :true
        !verify_ssl && uri.scheme == 'https'
      end

      def get(path, params = {})
        request_json :get, path, params
      end

      def post(path, params, discover = false)
        return request_json :post, path, params unless discover
        discover_request :post, path, params
      end

      def put(path, params)
        request_json :put, path, params
      end

      def delete(path)
        request_json :delete, path, {}
      end

      def validated(method, path, message = {}, discover = false)
        job_status = case method
                     when :post
                       post(path, message, discover)
                     when :put
                       put(path, message)
                     when :delete
                       delete(path)
                     end
        if job_status.code.match?(/2../)
          Puppet_X::EnterpriseModules::Ovm::JobValidator.new(
            job_status.body,
            self
          ).call
        else
          fail "Restfull call #{method} call to #{path} failed with status " \
               "#{job_status.code}."
        end
      end

      private

      def discover_request(method, path, params)
        request              = VERB_MAP[method.to_sym].new(path, MESSAGE_HEADERS)
        request.body         = params
        request.content_type = 'application/json'
        request.basic_auth(@username, @password)
        response = @http.request(request)
        error_handling(response, method, path) if error?(response)
        body = response.body ? JSON.parse(response.body) : {}
        OpenStruct.new(:code => response.code, :body => body)
      rescue JSON::ParserError
        response
      end

      def request_json(method, path, params)
        response = request(method, path, params)
        error_handling(response, method, path) if error?(response)
        body = response.body ? JSON.parse(response.body) : {}
        OpenStruct.new(:code => response.code, :body => body)
      rescue JSON::ParserError
        response
      end

      def authorization_password(options)
        decrypt(options.fetch(:password)) {fail 'You must pass a password'}
      end

      def error?(response)
        response.code[0] != '2'
      end

      def error_handling(response, method, path)
        fail "Error on #{method} to #{path}\nstatus #{response.code}, " \
             "message: #{JSON.parse(response.body)['message']}"
      end

      def valid_option?(option_key)
        VALID_OPTIONS.include?(option_key)
      end

      def all_valid_options?(options)
        options.keys.all? {|key| valid_option?(key)}
      end

      def option_fail_message(options)
        fail "options #{options} contains invalid key"
      end

      def request(method, path, params = {})
        request              = VERB_MAP[method.to_sym].new(path, MESSAGE_HEADERS)
        request.body         = params.to_json unless method == :get
        request.content_type = 'application/json'
        request.basic_auth(@username, @password)
        @http.request(request)
      end

      def encode_path_params(path, params)
        encoded = URI.encode_www_form(params)
        [path, encoded].join('?')
      end
    end
  end
end
