#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/connection'
require 'puppet_x/enterprisemodules/ovm/job_validator'

module Puppet_X
  module EnterpriseModules
    module Ovm
      module Data
        RESTFULL_TYPES = [
          'AccessGroup', 'AffinityGroup', 'ArchiveManagement', 'Assembly',
          'AssemblyVirtualDisk', 'AssemblyVm', 'BackupManagement',
          'BusinessManagement', 'Certificate', 'ClusterHeartbeatDevice',
          'Cluster', 'ClustetorageFs', 'ControlDomain',
          'CpuCompatibilityGroup', 'Cpu', 'EthernetPort', 'EventManagement',
          'Event', 'FileServerPlugin', 'FileServer', 'FileSystemMount',
          'FileSystem', 'JobManagement', 'Job', 'LogManagement',
          'MacManagement', 'Manager', 'ModelManagement', 'Network',
          'PeriodicTask', 'RasManagement', 'RepositoryExport', 'Repository',
          'ResourceGroup', 'ServerConfiguration', 'ServerController',
          'ServerPoolNetworkPolicy', 'ServerPoolPolicy', 'ServerPool',
          'Server', 'ServerUpdateConfiguration',
          'ServerUpdateRepositoryConfiguration', 'Statistic',
          'StatisticsManagement', 'StorageArrayPlugin', 'StorageArray',
          'StorageElement', 'StorageInitiator', 'StoragePath', 'StorageTarget',
          'UserPreference', 'VirtualDisk', 'VirtualNic', 'VirtualSwitch',
          'VlanInterface', 'VmCloneDefinition', 'VmCloneNetworkMapping',
          'VmCloneStorageMapping', 'VmDiskMapping', 'Vm', 'VolumeGroup', 'Zone'
        ].freeze

        def self.included(parent)
          parent.send(:include, Settings)
          parent.extend(Data)
        end

        def with_manager_connection(manager = nil)
          # Can be called from Type or Property
          mgr_name = if manager
                       manager
                     elsif is_a?(Puppet::Type)
                       manager_name
                     else
                       resource.manager_name
                     end
          options = options_for(mgr_name)
          connection = Puppet_X::EnterpriseModules::Connection.new(options)
          yield(connection)
        end

        RESTFULL_TYPES.each do |type|
          method_string = type.gsub(/(.)([A-Z])/, '\1_\2').downcase
          module_eval(<<-END_RUBY, __FILE__, __LINE__)
            #
            # Fetch the id of a specified type based on its name
            #
            def #{method_string}_id_for(name, manager: nil)
              id_for(type: '#{type}', name: name, manager: manager)
            end

            #
            # Fetch data for a specfied type. If the id is specfied, only for that
            # id. If no id is specfied, fetch all.
            #
            def #{method_string}_data(id = nil, manager: nil)
              data_for(type: '#{type}', id: id, manager: manager)
            end

            #
            # Fetch a reference (called an id in the OVM docs) based on the name
            # of the current type. This method expects the specified name to be part
            # of the type.
            #
            def #{method_string}_reference
              @#{method_string}_reference ||= #{method_string}_id_for(#{method_string}_name)
            end

            #
            # Return the pure raw id from the reference
            #
            def #{method_string}_id
              #{method_string}_reference['value']
            end

          END_RUBY
        end

        #
        # Get the id of the specified type based on the specified name
        #
        def id_for(type:, name:, manager: nil)
          data = data_for(type: type, manager: manager)
          id = data.find {|element| element['id']['name'] == name}
          fail "#{type} with name #{name} not found on server" if id.nil? || id['id'].nil?
          id['id']
        end

        #
        # Fetches data for specified type. When no id is specfied, we fetch all. When an id
        # is specfied, we fetch just the specfied record
        #
        def data_for(type:, id: nil, manager: nil)
          with_manager_connection(manager) do |connection|
            if id
              connection.get("/ovm/core/wsapi/rest/#{type}/#{id}").body
            else
              connection.get("/ovm/core/wsapi/rest/#{type}").body
            end
          end
        end

        #
        # Prepares hash with json id data of the object and values to be changed
        #
        def json_body(data, json)
          data.each { |k, v| json[camelback_key(k)] = v }
          json
        end

        def camelback_key(snake_key)
          parts = snake_key.to_s.split('_', 2)
          parts[0] << camelize(parts[1]) if parts.size > 1
          (parts[0] || '')
        end

        def options_for(manager)
          settings = configuration.fetch(manager)
          options = {
            :endpoint   => settings['manager_url'],
            :username   => settings['username'],
            :password   => settings['password'],
            :verify_ssl => settings['verify_ssl'],
            :port       => settings['port'],
          }
          options
        end
      end
    end
  end
end
