#
# See the file "LICENSE" for the full license governing this code.
#
module Puppet_X
  module EnterpriseModules
    module Ovm
      module Type
        Puppet_X::EnterpriseModules::Settings::SETTINGS_FILE = '/etc/ovm_manager.yaml'.freeze
        BASE_PATH = '/ovm/core/wsapi/rest'.freeze

        PUPPET_ATTRIBUTES = [
          :name, :alias, :audit, :before, :loglevel, :noop, :notify, :require,
          :schedule, :stage, :subscribe, :tag, :provider, :ensure
        ].freeze

        OVM_PUPPET_NAMES = [
            :access_group_name, :affinity_group_name, :archive_management_name,
            :assembly_name, :assembly_virtual_disk_name, :assembly_vm_name,
            :backup_management_name, :clone_definition_name,
            :clone_network_mapping_name, :clone_storage_mapping_name,
            :cluster_heartbeat_device_name, :cluster_name,
            :cluster_storage_name, :control_domain_name,
            :cpu_compatibility_group_name, :cpu_name, :ethernet_port_name,
            :event_management_name, :file_server_name, :file_server_plugin_name,
            :file_system_mount_name, :file_system_name, :isos,
            :job_management_name, :job_name, :log_management_name,
            :mac_management_name, :manager_name, :network_name, :ovm_tags,
            :periodic_task_name, :plugin, :repository_export_name,
            :repository_name, :rest_api_name, :server_controller_name,
            :server_id, :server_name, :server_pool_id, :server_pool_name,
            :server_pool_network_policy_name, :server_pool_policy_name,
            :server_update_configuration_name,
            :server_update_repository_configuration_name,
            :statistic_management_name, :storage_array_name,
            :storage_array_plugin_name, :storage_element_name,
            :storage_initiator_name, :storage_mapping_name, :storage_path_name,
            :storage_target_name, :tag_name, :urls, :virtual_disk_name,
            :virtual_disks, :virtual_disks, :virtual_nic_name, :vlan_name,
            :vm_disk_mapping_name, :vm_name, :volume_group_name
        ].freeze

        MANAGEMENT_TYPES = [
          'Puppet::Type::Ovm_archive_management',
          'Puppet::Type::Ovm_backup_management',
          'Puppet::Type::Ovm_event_management',
          'Puppet::Type::Ovm_job_management',
          'Puppet::Type::Ovm_log_management',
          'Puppet::Type::Ovm_mac_management',
          'Puppet::Type::Ovm_statistic_management'
        ].freeze

        def self.included(parent)
          parent.send(:include, EasyType)
          parent.send(:include, Settings)
          parent.send(:include, Data)
          parent.extend(ClassMethods)
          parent.extend(Puppet_X::EnterpriseModules::Ovm::TitleParser)
          parent.instance_eval { @exclude_properties = [] }
        end

        #
        # The payload puppet used to create the resource. Override this,
        # if you need to modify the create payload.
        #
        def create_payload
          payload = json_body(normalized_data, {})
          #
          # Find out if the property has a create_payload method.
          # If it has one, call it with the current state of the payload,
          # and allow it to transform the payload. The method MUST return the
          # modified payload.
          #
          property_methods(:create_payload, payload)
        end

        #
        # The Restfull path puppet used to create a resource. Override
        # this method when you need to manipulate the path
        #
        def create_path
          "#{BASE_PATH}/#{restfull_type}"
        end

        #
        # Code to be executed when Puppet finds that we need to create
        # a resource
        #
        def on_create
          Puppet.info "create #{name}"
          with_manager_connection do |connection|
            connection.validated(:post, create_path, create_payload)
          end
          nil
        end

        #
        # The payload puppet used to update the resource. Override this,
        # if you need to modify the create payload.
        #
        def update_payload
          payload = {}
          with_manager_connection do |connection|
            current_status = connection.get(update_path).body
            hash           = provider.property_flush.reject { |k, _v| exclude_properties.include?(k) }
            payload        = json_body(hash, current_status)
          end
          #
          # Find out if the property has a update_payload method.
          # If it has one, call it with the current state of the payload,
          # and allow it to transform the payload. The method MUST return the
          # modified payload.
          #
          property_methods(:update_payload, payload)
        end

        #
        # The restfull path puppet used to update a resource. Override
        # this method when you need to manipulate the path. If the type is the
        # management one, then return path similar to create_path.
        #
        def update_path
          return "#{BASE_PATH}/#{restfull_type}" if self.class.management_type?(self.class.to_s)
          "#{BASE_PATH}/#{restfull_type}/#{provider.id}"
        end

        #
        # Code to be executed when Puppet finds that we need to modify
        # a resource
        #
        def on_modify
          Puppet.info "modify #{name}"
          with_manager_connection do |connection|
            connection.validated(:put, update_path, update_payload)
          end

          nil
        end

        #
        # The restfull path puppet used to destroy a resource. Override
        # this method when you need to manipulate the path
        #
        def destroy_path
          "#{BASE_PATH}/#{restfull_type}/#{provider.id}"
        end

        #
        # Code to be executed when Puppet finds that we need to destroy
        # a resource
        #
        def on_destroy
          Puppet.info "destroy #{name}"
          with_manager_connection do |connection|
            connection.validated(:delete, destroy_path)
          end
          nil
        end

        #
        # Cleans puppet attributes from hash and sets name for Oracle VM
        # Convention is that the last one in the array of custom names is object
        # name.
        #
        def normalized_data
          hash        = to_hash.reject { |k, _v| PUPPET_ATTRIBUTES.include?(k) }
          hash        = hash.reject { |k, _v| OVM_PUPPET_NAMES.include?(k) }
          hash        = hash.reject { |k, _v| exclude_properties.include?(k) }
          attr_name   = self.class.key_attributes.last
          hash[:name] = self[attr_name]
          hash
        end

        #
        # Use connection information for specified manager and yield
        # it to the applied block
        #
        def with_manager_connection
          options = options_for(manager_name)
          connection = Puppet_X::EnterpriseModules::Connection.new(options)
          yield(connection)
        end

        #
        # Return the restfull type of this resource
        #
        def restfull_type
          self.class.restfull_type
        end

        private

        #
        # Find out if the property has a specified method.
        # If it has one, call it with the current state of the value
        # and allow it to transform the payload. The method MUST return the
        # modified value.
        #
        def property_methods(method, value)
          properties.each do |property|
            next unless property.respond_to?(method)
            value = property.send(method, value)
          end
          value
        end

        #
        # Prepares hash with json id data of the object and values to be changed
        #
        def json_body(data, json)
          data.each { |k, v| json[camelback_key(k)] = v }
          json
        end

        #
        # translate all keys of the type into camelized keys
        # used for the update payload
        #
        def camelback_key(snake_key)
          parts = snake_key.to_s.split('_', 2)
          parts[0] << camelize(parts[1]) if parts.size > 1
          (parts[0] || '')
        end

        #
        # Get all information for a specified OVM manager
        #
        def options_for(manager)
          settings = configuration.fetch(manager)
          {
            :endpoint   => settings['manager_url'],
            :username   => settings['username'],
            :password   => settings['password'],
            :verify_ssl => settings['verify_ssl'],
            :port       => settings['port']
          }
        end

        #
        # Get the properties we don't want to include in the update_payload
        #
        def exclude_properties
          self.class.instance_variable_get(:'@exclude_properties')
        end

        module ClassMethods
          def module_name
            'ovm_config'
          end

          #
          # Construct the path we use to fetch all resources
          #
          def index_path
            "#{BASE_PATH}/#{restfull_type}"
          end

          #
          # Fetches all raw resources through the restful api and provide the
          # digestable data for Puppet
          #
          def get_raw_resources
            resources = get_api_resources
            return resources.flatten unless @resource_filter
            resources.flatten.find_all { |c| @resource_filter.call(c) }
          end

          #
          # Calls API and gets current resources for the type with manager added
          #
          def get_api_resources
            api_resources = []
            configuration.keys.collect do |manager|
              with_manager_connection(manager) do |connection|
                elements = process_elements(connection.get(index_path).body)
                fail "Internal error when fetching all api_resources on #{index_path}." if elements == ''
                api_resources << elements_with_manager(elements, manager)
              end
            end
            api_resources
          end

          #
          # Package elements into array if they are in the form of hash and not
          # array of hashes
          #
          def process_elements(elements)
            return elements unless management_type?(to_s) && !elements.is_a?(Array)
            [elements]
          end

          #
          # Add current manager to resource elements
          #
          def elements_with_manager(elements, manager)
            elements.collect { |m| m.merge('manager' => manager) }
          end

          #
          # Based on the value of ensure, do an autorequire or an autobefore.
          #
          def child_of(type, variable = nil, &proc)
            #
            # If the type is "management" then just do autorequire and go to
            # on_modify
            #
            return management_type_auto_require(type, &proc) if management_type?(to_s)
            standard_autorequire(type, variable, &proc)
            standard_autobefore(type, variable, &proc)
          end

          def management_type_auto_require(type, &proc)
            autorequire(type) do
              instance_eval(&proc)
            end
          end

          def standard_autorequire(type, variable, &proc)
            autorequire(type) do
              present = instance_eval('self[:ensure]', __FILE__, __LINE__ + 1) == :present
              if variable.nil? && present # Always
                instance_eval(&proc)
              elsif present && self[variable].nil? || self[variable].to_s == 'absent'
                instance_eval(&proc)
              end
            end
          end

          def standard_autobefore(type, variable, &proc)
            autobefore(type) do
              absent = instance_eval('self[:ensure]', __FILE__, __LINE__ + 1) == :absent
              if absent && variable.nil? # Always
                instance_eval(&proc)
                #
                # Check the value of the provider is set
                #
              elsif absent && provider
                full_resource(provider).instance_eval(&proc) if provider.send(variable) != :absent
              end
            end
          end

          #
          # Set or fetch the Restfull type fetch the resources
          #
          def restfull_type(restfull_type = nil)
            @restfull_type = restfull_type ? restfull_type : @restfull_type
          end

          #
          # Make sure these properties are not included from the
          # type to construct the update_payload
          #
          def exclude_properties(*properties)
            @exclude_properties = *properties
          end

          #
          # Set the filter for the found raw_resources
          #
          def resource_filter(&filter)
            @resource_filter = filter
          end

          #
          # Checks if the object klass name is the management type. They come as a
          # single hash with values. This is specific for "manager" rest classes
          #
          def management_type?(klass_name)
            MANAGEMENT_TYPES.include?(to_s)
          end
        end
      end
    end
  end
end
